@@ -2,16 +2,17 @@
 from flask import current_app, g, request, abort
 from flask_login import current_user
 from itsdangerous import exc
-from jinja2 import TemplateSyntaxError
+from jinja2 import TemplateNotFound, TemplateSyntaxError
 from jinja2.ext import Extension
 from jinja2.nodes import List, CallBlock
-from esentity.models import Page
+from esentity.models import Page, Activity
 from loguru import logger
 from json import dumps, loads
 import html
 import math
 from urllib.parse import unquote
 from babel.numbers import format_decimal
+import pendulum
 
 
 class HtmlExtension(Extension):
@@ -33,252 +34,364 @@ class HtmlExtension(Extension):
         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)
 
 
-class LicencesExtension(Extension):
-    tags = set(['licences'])
+# class LicencesExtension(Extension):
+#     tags = set(['licences'])
+
+#     def __init__(self, environment):
+#         super(LicencesExtension, self).__init__(environment)
+
+#     def _render(self, caller):
+#         pages, _ = Page.get(
+#             category='collection',
+#             is_active=True,
+#             is_searchable=True,
+#             is_redirect=False,  
+#             locale=g.language, 
+#             _source=['title', 'path', 'alt_title'],
+#             collection_mode='CasinoLicence',
+#             _count=100,
+#             _sort=[
+#                 {'alt_title.keyword': {'order': 'asc'}}
+#             ]
+#         )
+#         t = current_app.jinja_env.get_template('_table-licences.html')
+#         s = t.render(pages=pages)
+#         return s
+
+#     def parse(self, parser):
+#         lineno = next(parser.stream).lineno
+
+#         body = ''
+#         args = []
+
+#         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)
+
+
+class CollectionsExtension(Extension):
+    tags = set(['collections'])
 
     def __init__(self, environment):
-        super(LicencesExtension, self).__init__(environment)
+        super(CollectionsExtension, self).__init__(environment)
 
-    def _render(self, caller):
-        pages, _ = Page.get(
+    def _render(self, args, caller=None):
+        logger.info('collections Args IN: {0}'.format(args))
+
+        default = ['collections', 'CasinoSoftware', 100, None, 500, []]
+        _ca = len(default)
+        args = args[:_ca] + default[len(args):] + args[_ca:]
+
+        logger.info('collections Args OUT: {0}'.format(args))
+
+        template = args[0]
+        mode = args[1]
+        count = int(args[2])
+        aggs_key = args[3]
+        aggs_count = int(args[4])
+        exclude = args[5]
+
+        items, _ = Page.get(
             category='collection',
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
             locale=g.language, 
-            _source=['title', 'path', 'alt_title'],
-            collection_mode='CasinoLicence',
-            _count=100,
+            _source=['title', 'path', 'alt_title', 'custom_text', 'logo', 'theme_color'],
+            collection_mode=mode,
+            _count=count,
+            _exclude=exclude,
             _sort=[
                 {'alt_title.keyword': {'order': 'asc'}}
             ]
         )
-        t = current_app.jinja_env.get_template('_table-licences.html')
-        s = t.render(pages=pages)
-        return s
 
-    def parse(self, parser):
-        lineno = next(parser.stream).lineno
-
-        body = ''
-        args = []
-
-        return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)
-
-
-class SoftwaresExtension(Extension):
-    tags = set(['softwares'])
-
-    def __init__(self, environment):
-        super(SoftwaresExtension, self).__init__(environment)
-
-    def _render(self, caller):
-        pages, _ = Page.get(
-            category='collection',
-            is_active=True,
-            is_searchable=True,
-            is_redirect=False,  
-            locale=g.language, 
-            _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
-            collection_mode='CasinoSoftware',
-            _count=100,
-            _sort=[
-                {'alt_title.keyword': {'order': 'asc'}}
+        aggs = []
+        if aggs_key:
+            filter_list =  [ 
+                {"term": {"is_active": True}},
+                {"term": {"is_draft": False}},
+                {"term": {"is_sandbox": False}},
+                {"term": {"is_searchable": True}},
+                {"term": {"is_redirect": False}},
+                {"term": {"category.keyword": 'provider'}},
+                {"term": {"locale.keyword": g.language}},
             ]
-        )
-        pages = {item.alt_title: item for item in pages}
 
-        # v2 (with aggs)
-
-        filter_list =  [ 
-            {"term": {"is_active": True}},
-            {"term": {"is_draft": False}},
-            {"term": {"is_sandbox": False}},
-            {"term": {"is_searchable": True}},
-            {"term": {"is_redirect": False}},
-            {"term": {"category.keyword": 'provider'}},
-            {"term": {"locale.keyword": g.language}},
-        ]
-
-        _aggs = {
-            "software": {
-                "terms": {
-                    "field": "software.keyword",
-                    "size": 500,
-                    "order": {"_count": "desc"}
-                },
-                # "aggs": {
-                #     "casinos": {
-                #         "top_hits": {
-                #             "sort": [
-                #                 {
-                #                     "rating": {
-                #                         "order": "desc"
-                #                     }
-                #                 }
-                #             ],
-                #             "_source": {
-                #                 "includes": ["title", "path", "alt_title", "rating"]
-                #             },
-                #             "size": 3
-                #         }
-                #     }                
-                # }
-            }
-        }
-
-        q = {
-            "query": {
-                "bool": {
-                    "must": filter_list,
+            _aggs = {
+                aggs_key: {
+                    "terms": {
+                        "field": f"{aggs_key}.keyword",
+                        "size": aggs_count,
+                        "order": {"_count": "desc"}
+                    },
+                    # "aggs": {
+                    #     "casinos": {
+                    #         "top_hits": {
+                    #             "sort": [
+                    #                 {
+                    #                     "rating": {
+                    #                         "order": "desc"
+                    #                     }
+                    #                 }
+                    #             ],
+                    #             "_source": {
+                    #                 "includes": ["title", "path", "alt_title", "rating"]
+                    #             },
+                    #             "size": 3
+                    #         }
+                    #     }                
+                    # }
                 }
-            },
-            "aggs": _aggs,
-            "size": 0,
-            "_source": False,
-        }
-
-        resp = current_app.es.search(index=Page._table(), body=q, request_timeout=60, ignore=[400, 404])
-
-        if 'status' in resp and resp['status'] in [400, 404]:
-            return ''
-
-        t = current_app.jinja_env.get_template('_table-simple.html')
-        s = t.render(pages=pages, title='Software', items=resp['aggregations']['software']['buckets'])
-
-        return s
-
-    def parse(self, parser):
-        lineno = next(parser.stream).lineno
-
-        body = ''
-        args = []
-
-        return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)
-
-
-class CountriesExtension(Extension):
-    tags = set(['countries'])
-
-    def __init__(self, environment):
-        super(CountriesExtension, self).__init__(environment)
-
-    def _render(self, caller):
-        pages, _ = Page.get(
-            category='collection',
-            is_active=True,
-            is_searchable=True,
-            is_redirect=False,  
-            locale=g.language, 
-            _source=['title', 'path', 'alt_title'],
-            collection_mode='CasinoCountry',
-            _count=100,
-            _sort=[
-                {'alt_title.keyword': {'order': 'asc'}}
-            ]
-        )
-        t = current_app.jinja_env.get_template('_table-countries.html')
-        s = t.render(pages=pages)
-        return s
-
-    def parse(self, parser):
-        lineno = next(parser.stream).lineno
-
-        body = ''
-        args = []
-
-        return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)        
-
-
-class BankingExtension(Extension):
-    tags = set(['banking'])
-
-    def __init__(self, environment):
-        super(BankingExtension, self).__init__(environment)
-
-    def _render(self, caller):
-        pages, _ = Page.get(
-            category='collection',
-            is_active=True,
-            is_searchable=True,
-            is_redirect=False,  
-            locale=g.language, 
-            _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
-            collection_mode=['CasinoBanking', 'CasinoBankingLimits'],
-            _count=100,
-            _sort=[
-                {'alt_title.keyword': {'order': 'asc'}}
-            ]
-        )
-        pages = {item.alt_title: item for item in pages}
-
-        # v2 (with aggs)
-
-        filter_list =  [ 
-            {"term": {"is_active": True}},
-            {"term": {"is_draft": False}},
-            {"term": {"is_sandbox": False}},
-            {"term": {"is_searchable": True}},
-            {"term": {"is_redirect": False}},
-            {"term": {"category.keyword": 'provider'}},
-            {"term": {"locale.keyword": g.language}},
-        ]
-
-        _aggs = {
-            "deposits": {
-                "terms": {
-                    "field": "deposits.keyword",
-                    "size": 500,
-                    "order": {"_count": "desc"}
-                },
-                # "aggs": {
-                #     "casinos": {
-                #         "top_hits": {
-                #             "sort": [
-                #                 {
-                #                     "rating": {
-                #                         "order": "desc"
-                #                     }
-                #                 }
-                #             ],
-                #             "_source": {
-                #                 "includes": ["title", "path", "alt_title", "rating"]
-                #             },
-                #             "size": 3
-                #         }
-                #     }                
-                # }
             }
-        }
 
-        q = {
-            "query": {
-                "bool": {
-                    "must": filter_list,
-                }
-            },
-            "aggs": _aggs,
-            "size": 0,
-            "_source": False,
-        }
+            q = {
+                "query": {
+                    "bool": {
+                        "must": filter_list,
+                    }
+                },
+                "aggs": _aggs,
+                "size": 0,
+                "_source": False,
+            }
 
-        resp = current_app.es.search(index=Page._table(), body=q, request_timeout=60, ignore=[400, 404])
+            resp = current_app.es.search(index=Page._table(), body=q, request_timeout=60, ignore=[400, 404])
+            aggs = resp['aggregations'][aggs_key]['buckets']
 
-        if 'status' in resp and resp['status'] in [400, 404]:
-            return ''
+        logger.info(aggs)
 
-        t = current_app.jinja_env.get_template('_table-simple.html')
-        s = t.render(pages=pages, title='Payment Method', items=resp['aggregations']['deposits']['buckets'])
+        t = current_app.jinja_env.get_template(f'_ext-{template}.html')
+        s = t.render(
+            items=items, 
+            aggs={item['key']: item['doc_count'] for item in aggs}
+        )
 
         return s
 
+
     def parse(self, parser):
         lineno = next(parser.stream).lineno
 
-        body = ''
         args = []
+        while parser.stream.current.type != 'block_end':
+            args.append(parser.parse_expression())
+            parser.stream.skip_if('comma')
 
-        return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)  
+        return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
+
+
+# class CollectionsExtension(Extension):
+#     tags = set(['collections'])
+
+#     def __init__(self, environment):
+#         super(CollectionsExtension, self).__init__(environment)
+
+#     def _render(self, caller):
+#         pages, _ = Page.get(
+#             category='collection',
+#             is_active=True,
+#             is_searchable=True,
+#             is_redirect=False,  
+#             locale=g.language, 
+#             _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
+#             collection_mode='CasinoSoftware',
+#             _count=100,
+#             _sort=[
+#                 {'alt_title.keyword': {'order': 'asc'}}
+#             ]
+#         )
+#         pages = {item.alt_title: item for item in pages}
+
+#         # v2 (with aggs)
+
+#         filter_list =  [ 
+#             {"term": {"is_active": True}},
+#             {"term": {"is_draft": False}},
+#             {"term": {"is_sandbox": False}},
+#             {"term": {"is_searchable": True}},
+#             {"term": {"is_redirect": False}},
+#             {"term": {"category.keyword": 'provider'}},
+#             {"term": {"locale.keyword": g.language}},
+#         ]
+
+#         _aggs = {
+#             "software": {
+#                 "terms": {
+#                     "field": "software.keyword",
+#                     "size": 500,
+#                     "order": {"_count": "desc"}
+#                 },
+#                 # "aggs": {
+#                 #     "casinos": {
+#                 #         "top_hits": {
+#                 #             "sort": [
+#                 #                 {
+#                 #                     "rating": {
+#                 #                         "order": "desc"
+#                 #                     }
+#                 #                 }
+#                 #             ],
+#                 #             "_source": {
+#                 #                 "includes": ["title", "path", "alt_title", "rating"]
+#                 #             },
+#                 #             "size": 3
+#                 #         }
+#                 #     }                
+#                 # }
+#             }
+#         }
+
+#         q = {
+#             "query": {
+#                 "bool": {
+#                     "must": filter_list,
+#                 }
+#             },
+#             "aggs": _aggs,
+#             "size": 0,
+#             "_source": False,
+#         }
+
+#         resp = current_app.es.search(index=Page._table(), body=q, request_timeout=60, ignore=[400, 404])
+
+#         if 'status' in resp and resp['status'] in [400, 404]:
+#             return ''
+
+#         t = current_app.jinja_env.get_template('_table-simple.html')
+#         s = t.render(pages=pages, title='Software', items=resp['aggregations']['software']['buckets'])
+
+#         return s
+
+#     def parse(self, parser):
+#         lineno = next(parser.stream).lineno
+
+#         body = ''
+#         args = []
+
+#         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)
+
+
+# class CountriesExtension(Extension):
+#     tags = set(['countries'])
+
+#     def __init__(self, environment):
+#         super(CountriesExtension, self).__init__(environment)
+
+#     def _render(self, caller):
+#         pages, _ = Page.get(
+#             category='collection',
+#             is_active=True,
+#             is_searchable=True,
+#             is_redirect=False,  
+#             locale=g.language, 
+#             _source=['title', 'path', 'alt_title'],
+#             collection_mode='CasinoCountry',
+#             _count=100,
+#             _sort=[
+#                 {'alt_title.keyword': {'order': 'asc'}}
+#             ]
+#         )
+#         t = current_app.jinja_env.get_template('_table-countries.html')
+#         s = t.render(pages=pages)
+#         return s
+
+#     def parse(self, parser):
+#         lineno = next(parser.stream).lineno
+
+#         body = ''
+#         args = []
+
+#         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)        
+
+
+# class BankingExtension(Extension):
+#     tags = set(['banking'])
+
+#     def __init__(self, environment):
+#         super(BankingExtension, self).__init__(environment)
+
+#     def _render(self, caller):
+#         pages, _ = Page.get(
+#             category='collection',
+#             is_active=True,
+#             is_searchable=True,
+#             is_redirect=False,  
+#             locale=g.language, 
+#             _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
+#             collection_mode=['CasinoBanking', 'CasinoBankingLimits'],
+#             _count=100,
+#             _sort=[
+#                 {'alt_title.keyword': {'order': 'asc'}}
+#             ]
+#         )
+#         pages = {item.alt_title: item for item in pages}
+
+#         # v2 (with aggs)
+
+#         filter_list =  [ 
+#             {"term": {"is_active": True}},
+#             {"term": {"is_draft": False}},
+#             {"term": {"is_sandbox": False}},
+#             {"term": {"is_searchable": True}},
+#             {"term": {"is_redirect": False}},
+#             {"term": {"category.keyword": 'provider'}},
+#             {"term": {"locale.keyword": g.language}},
+#         ]
+
+#         _aggs = {
+#             "deposits": {
+#                 "terms": {
+#                     "field": "deposits.keyword",
+#                     "size": 500,
+#                     "order": {"_count": "desc"}
+#                 },
+#                 # "aggs": {
+#                 #     "casinos": {
+#                 #         "top_hits": {
+#                 #             "sort": [
+#                 #                 {
+#                 #                     "rating": {
+#                 #                         "order": "desc"
+#                 #                     }
+#                 #                 }
+#                 #             ],
+#                 #             "_source": {
+#                 #                 "includes": ["title", "path", "alt_title", "rating"]
+#                 #             },
+#                 #             "size": 3
+#                 #         }
+#                 #     }                
+#                 # }
+#             }
+#         }
+
+#         q = {
+#             "query": {
+#                 "bool": {
+#                     "must": filter_list,
+#                 }
+#             },
+#             "aggs": _aggs,
+#             "size": 0,
+#             "_source": False,
+#         }
+
+#         resp = current_app.es.search(index=Page._table(), body=q, request_timeout=60, ignore=[400, 404])
+
+#         if 'status' in resp and resp['status'] in [400, 404]:
+#             return ''
+
+#         t = current_app.jinja_env.get_template('_table-simple.html')
+#         s = t.render(pages=pages, title='Payment Method', items=resp['aggregations']['deposits']['buckets'])
+
+#         return s
+
+#     def parse(self, parser):
+#         lineno = next(parser.stream).lineno
+
+#         body = ''
+#         args = []
+
+#         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)  
 
 
 class RatingExtension(Extension):
@@ -290,7 +403,7 @@ class RatingExtension(Extension):
     def _render(self, args, caller=None):
         logger.info(f'Casinos Args IN: {args}')
 
-        default = ['casino', True, 10]
+        default = ['casino', True, 10, False]
         _ca = len(default)
         args = args[:_ca] + default[len(args):] + args[_ca:]
 
@@ -299,8 +412,9 @@ class RatingExtension(Extension):
         service = args[0]
         has_filters = args[1]
         count = int(args[2])
+        row_mode = bool(args[3])
 
-        tags = args[3:]
+        tags = args[4:]
         logger.info('Casinos Tags: {0}'.format(tags))
 
         _afields = ['software', 'licences', 'deposits', 'withdrawal', 'games']
@@ -367,6 +481,7 @@ class RatingExtension(Extension):
             count=count,
             tags=tags,
             deposits_primary=deposits_primary,
+            row_mode=row_mode
         )
         return s
 
@@ -390,7 +505,7 @@ class TopSlotsExtension(Extension):
     def _render(self, args, caller=None):
         logger.info('Top Slots IN Args: {0}'.format(args))
 
-        default = ['NetEnt', 8, 4, False]
+        default = ['NetEnt', 8, 4, False, True]
         args = args[:4] + default[len(args):] + args[4:]
 
         logger.info('Top Slots OUT Args: {0}'.format(args))
@@ -399,6 +514,7 @@ class TopSlotsExtension(Extension):
         count = int(args[1])
         in_row = int(args[2])
         random = bool(args[3])
+        show_provider = bool(args[4])
 
         # args: service
         slots, found = Page.get(
@@ -417,6 +533,7 @@ class TopSlotsExtension(Extension):
         s = t.render(
             slots=slots,
             in_row=in_row,
+            show_provider=show_provider,
         )
         return s
 
@@ -640,22 +757,23 @@ class CasinosExtension(Extension):
         super(CasinosExtension, self).__init__(environment)
 
     def _render(self, args, caller=None):
-        logger.info('Casinos Args IN: {0}'.format(args))
+        logger.info('casinos Args IN: {0}'.format(args))
 
-        default = [10, 'casinos_top', False] # cnt, template, has_filters, [tags...]
+        default = [10, 'casinos_top', True, None] # cnt, template, has_geo, sort, [tags...]
         _ca = len(default)
         args = args[:_ca] + default[len(args):] + args[_ca:]
 
-        logger.info('Casinos Args OUT: {0}'.format(args))
+        logger.info('casinos Args OUT: {0}'.format(args))
 
         count = int(args[0])
         template = args[1]
-        has_filters = args[2]
+        has_geo = bool(args[2])
+        sorting = args[3]
 
-        tags = args[3:]
-        logger.info('Casinos Tags: {0}'.format(tags))
+        tags = args[len(default):]
+        logger.info('casinos Tags: {0}'.format(tags))
 
-        if has_filters:
+        if False:
             pass
             # _afields = ['software', 'licences', 'deposits', 'withdrawal', 'games']
             # _aggs = {
@@ -687,25 +805,28 @@ class CasinosExtension(Extension):
             kwargs = {
                 'is_searchable': True,
                 'is_redirect': False,
-                'country': current_user.country_full,
-                'services': 'casino',
+                'country': current_user.country_full if has_geo else None,
+                # 'services': 'casino',
                 'provider_tags': tags,
                 '_source': True, 
                 '_count': count,
+                '_sorting': sorting,
             }
             if len(current_app.config['AVAILABLE_LOCALE']) > 1:
                 kwargs['locale_available'] = g.language
 
             pages, found = Page.provider_by_context(**kwargs)
+            logger.info(f'Casinos found: {found}')
+
+        now = pendulum.now('UTC')
 
         t = current_app.jinja_env.get_template('_ext-{0}.html'.format(template))
         s = t.render(
             pages=pages, 
             found=found, 
-            has_filters=has_filters, 
+            # has_filters=has_filters, 
             count=count,
-            # id=id, 
-            # tags=tags,
+            my=now.format('MMMM YYYY'),
         )
         return s
 
@@ -771,7 +892,7 @@ class SlotsExtension(Extension):
             _c = dumps(aggs)
             current_app.redis.hset('aggs', id, _c)
             logger.info('Cache aggs {0}: {1}'.format(id, len(_c)))
-            
+
         t = current_app.jinja_env.get_template('_ext-{0}.html'.format(template))
         s = t.render(
             slots=slots, 
@@ -1558,11 +1679,17 @@ class UIExtension(Extension):
             def parse_item(s):
                 k, v = s.split(':')
                 return {'key': k, 'value': v}
-            items = [parse_item(item) for item in rv.split('|')]
+            try:
+                items = [parse_item(item) for item in rv.split('|')]
+            except:
+                logger.warning(f'ui extension parse error: {rv}')
 
-        t = current_app.jinja_env.get_template(f'_ext-ui_{template}.html')
-        # html.unescape(rv)
-        s = t.render(items=items, rv=rv, args=args)
+        s = rv
+        try:
+            t = current_app.jinja_env.get_template(f'_ext-ui_{template}.html')
+            s = t.render(items=items, rv=html.unescape(rv), args=args)
+        except TemplateNotFound:
+            pass
 
         return s
 
@@ -1583,23 +1710,137 @@ class UIExtension(Extension):
         return CallBlock(callback, [], [], body).set_lineno(tag.lineno)
 
 
-class ComplaintsStatsExtension(Extension):
-    tags = set(['complaints_stats'])
+class ComplaintsExtension(Extension):
+    tags = set(['complaints'])
 
     def __init__(self, environment):
-        super(ComplaintsStatsExtension, self).__init__(environment)
+        super(ComplaintsExtension, self).__init__(environment)
 
-    def _render(self, caller):
-        stats = {}
-        try:
-            stats = loads(current_app.redis.get('gs:complaints_stats').decode('utf-8'))
-        except Exception:
-            pass
+    def _render(self, args, caller=None):
+        logger.info(f'ComplaintsExtension Args IN: {args}')
 
-        t = current_app.jinja_env.get_template('_ext-complaint_stats.html')
-        s = t.render(stats=stats)
+        default = [10]
+        _ca = len(default)
+        args = args[:_ca] + default[len(args):] + args[_ca:]
+
+        logger.info('ComplaintsExtension Args OUT: {0}'.format(args))
+
+        count = int(args[0])
+
+        complaints, total = Activity.get(
+            activity='complaint',
+            is_active=True,
+            status=['opened', 'not_solved', 'solved', 'rejected'],
+            _sort=[{'createdon': 'desc'}],
+            _count=count,
+        ) 
+
+        casinos, _ = Page.get(
+            category='provider',
+            _all=True,
+            _process=False,
+            _count=1000,
+            _source=[
+                'path', 
+            ]
+        )
+        casinos = {item['_id']: item['_source']['path'] for item in casinos}
+
+        t = current_app.jinja_env.get_template('_ext-complaints.html')
+        s = t.render(
+            items=complaints, 
+            cpp=count,
+            total=total,
+            count=count,
+            percent=int((count / total) * 100) if total else 0,
+            casinos=casinos,
+        )
         return s
 
     def parse(self, parser):
         lineno = next(parser.stream).lineno
-        return CallBlock(self.call_method("_render", []), [], [], '').set_lineno(lineno)
\ No newline at end of file
+
+        args = []
+        while parser.stream.current.type != 'block_end':
+            args.append(parser.parse_expression())
+            parser.stream.skip_if('comma')
+
+        return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
+
+
+def process_aggs(_a, _k, _t):
+    if _k in _a:
+        _r = []
+        for _i in _a[_k]:
+            if _i['item'] in _t:
+                _r.append(_i)
+        _a[_k] = _r
+    return _a
+
+class RatingV2Extension(Extension):
+    tags = set(['rating'])
+
+    def __init__(self, environment):
+        super(RatingV2Extension, self).__init__(environment)
+
+    def _render(self, args, caller=None):
+        logger.info(f'rating_v2 Args IN: {args}')
+
+        default = ['casino', 10, ['software', 'provider_tags', 'currencies', 'deposits', 'games'], 1000, ['Fair', 'Anomymous', 'VPN']]
+        _ca = len(default)
+        args = args[:_ca] + default[len(args):] + args[_ca:]
+
+        logger.info('rating_v2 Args OUT: {0}'.format(args))
+
+        service = args[0]
+        count = int(args[1])
+        aggs_fields = args[2]
+        aggs_count = int(args[3])
+        tags_enabled = args[4]
+
+        tags = args[len(default):]
+        logger.info('Casinos Tags: {0}'.format(tags))
+
+        _aggs = {
+            item: {
+                "terms": {
+                    "field": "{0}.keyword".format(item),
+                    "size": aggs_count,
+                    "order": {"_key": "asc"}
+                }
+            } for item in aggs_fields
+        }
+
+        # args: service
+        pages, found, aggs, _ = Page.provider_by_context(
+            is_searchable=True,
+            is_redirect=False,
+            country=current_user.country_full,
+            services=service,
+            provider_tags=tags,
+            _locale=g.language,
+            _source=True,
+            _count=count,
+            _aggs=_aggs,
+        )
+
+        t = current_app.jinja_env.get_template('_ext-rating_casinos.html')
+        s = t.render(
+            pages=pages, 
+            found=f'{found} matching casinos', 
+            service=service, 
+            count=count,
+            tags=tags,
+            aggs=process_aggs(aggs, 'provider_tags', tags_enabled),
+        )
+        return s
+
+    def parse(self, parser):
+        lineno = next(parser.stream).lineno
+
+        args = []
+        while parser.stream.current.type != 'block_end':
+            args.append(parser.parse_expression())
+            parser.stream.skip_if('comma')
+
+        return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
